try:
    import pygame
    import random
    import os
    import math
    from pygame.locals import *
except ImportError, bad:
    print 'NO LOAD %s OK?' % (bad)

def load_graphics(file):
    '''Loads image files'''
    path = os.path.join('data/graphics', file)
    image = pygame.image.load(path)
    image.set_colorkey((255, 0, 255))
    image = image.convert()
    return image, image.get_rect()

class GameBall(pygame.sprite.Sprite):
    '''The ball moving across the screen that you play with.'''
    def __init__(self, xy, vector):
        pygame.sprite.Sprite.__init__(self)
        self.image, self.rect = load_graphics('ball.png')
        window = pygame.display.get_surface()
        self.area = window.get_rect()
        self.vector = vector
        self.impact = 0
        self.rect.topleft = xy

    def update(self):
        self.rect = self.newloc(self.rect, self.vector)

        if not self.area.contains(self.rect):
            topleft = not self.area.collidepoint(self.rect.topleft)
            topright = not self.area.collidepoint(self.rect.topright)
            bottomleft = not self.area.collidepoint(self.rect.bottomleft)
            bottomright = not self.area.collidepoint(self.rect.bottomright)

            if (topright and topleft) or (bottomright and bottomleft):
                self.vector[0] = -self.vector[0]
            if topleft and bottomleft:
		self.score(2)
            if topright and bottomright:
                self.score(1)
		
        else:
            firstplayer.rect.inflate(-2, -2)
            secondplayer.rect.inflate(-2, -2)

            if self.rect.colliderect(firstplayer.rect) == 1 and not self.impact:
                self.vector[0] = math.pi - self.vector[0]
                self.impact = not self.impact
            elif self.rect.colliderect(secondplayer.rect) == 1 and not self.impact:
                self.vector[0] = math.pi - self.vector[0]
                self.impact = not self.impact
            elif self.impact:
                self.impact = not self.impact
        
    def newloc(self,rect,vector):
		angle = self.vector[0]
                hyp = self.vector[1]
		(newx,newy) = (hyp*math.cos(angle),hyp*math.sin(angle))
		return rect.move(newx,newy)

    def score(self, player):
        if player == 1:
            secondplayer.increasepoints(1)
        else:
            firstplayer.increasepoints(1)

        scoreboard()
        text = 'PLAYER %s BEAT YOU SUCKA' % player
        font = pygame.font.Font(None, 50)
        ren = font.render(text, 1, (150, 150, 200))
        pos = ren.get_rect()
        pos.centerx = background.get_rect().centerx
        pos.centery = 300
        window.blit(ren, pos)
        pygame.display.flip()
        pygame.time.wait(800)
        window.blit(background, (0,0))

        scoreboard()
        self.rect.center = (400,300) #self.area.center
	self.vector[0] = random.uniform(0, 2*math.pi)
	firstplayer.reinit()
	secondplayer.reinit()

class PlayerBat(pygame.sprite.Sprite):
    '''User-controlled bat'''
    def __init__(self, field):
        pygame.sprite.Sprite.__init__(self)
        self.image, self.rect = load_graphics('bat.png')
        window = pygame.display.get_surface()
        self.area = window.get_rect()
        self.field = field
        self.points = 0
        self.reinit()

    def reinit(self):
        self.accel = [0.0,0.5]
        self.vel = [0.0,0.0]
        if self.field == 'left':
            self.rect.midleft = self.area.midleft
            self.rect = self.rect.move(10,0)
        elif self.field == 'right':
            self.rect.midright = self.area.midright
            self.rect = self.rect.move(-10,0)

    def update(self):
        self.vel[0] += self.accel[0]
        self.vel[1] += self.accel[1]
        newloc = self.rect.move(self.vel)
        if self.area.contains(newloc): #keeps the bat from going off-screen
            self.rect = newloc
        else:
            self.vel[1] /= -3
                
        pygame.event.pump()

    def lift(self):
        self.accel = [0.0, -0.25]

    def increasepoints(self, score):
        self.points += score

class Steam(pygame.sprite.Sprite):
    def __init__(self):
        pygame.sprite.Sprite.__init__(self)
        self.image, self.rect = load_graphics('steam.png')
        window = pygame.display.get_surface()
        self.area = window.get_rect()

    def steamy(self):
        for player in [firstplayer, secondplayer]:
            block = player.rect

            if player.accel[1] <0:
                for y in range(block.top+block.height, 600, 30):
                    window.blit(self.image, pygame.Rect(block.left - 11/2, y, 0, 0))
            
def scoreboard(): #Keeps track of the points scored
    font = pygame.font.Font(None, 108)
    text = str(firstplayer.points)
    scored1 = font.render(text, 1, (108,108,108))

    font = pygame.font.Font(None, 108)
    text = str(secondplayer.points)
    scored2 = font.render(text, 1, (108,108,108))

    window.blit(background, scored1.get_rect())
    window.blit(background, scored1.get_rect())
    window.blit(scored1, (750, 10))
    window.blit(scored2, (10, 10))
    

def main():
    pygame.init()
    global window
    window = pygame.display.set_mode((800,600))
    pygame.display.set_caption('SteamPong')
    pygame.mouse.set_visible(0)
    
    global background
    background = pygame.Surface(window.get_size())
    background = background.convert()
    background.fill((50, 50, 50))

    global firstplayer
    global secondplayer
    firstplayer = PlayerBat('left')
    secondplayer = PlayerBat('right')
    clock = pygame.time.Clock()
    steam = Steam()

    global speed
    speed = 7
    global gameball
    gameball = GameBall((400, 300),[random.randint(1, 10), speed])

    peoplesprites = pygame.sprite.RenderPlain((firstplayer, secondplayer))
    spriteball = pygame.sprite.RenderPlain(gameball)

    window.blit(background, (0, 0))
    pygame.display.flip()
    
    while 1:
        for event in pygame.event.get():
            if event.type == QUIT:
                return
            
            elif event.type == KEYDOWN:
                if event.key == K_UP:
                    secondplayer.lift()
                if event.key == K_w:
                    firstplayer.lift()
                if event.key == K_ESCAPE:
                    return
                
            if event.type == KEYUP:
                if event.key == K_UP:
                   secondplayer.accel = [0.0, 0.5]
                if event.key == K_w:
                   firstplayer.accel = [0.0, 0.5]
     
        clock.tick(60)
        
        window.blit(background, gameball.rect, gameball.rect)
        window.blit(background, firstplayer.rect, firstplayer.rect)
        window.blit(background, secondplayer.rect, secondplayer.rect)
        scoreboard()
        steam.steamy()
        spriteball.update()
        peoplesprites.update()
        spriteball.draw(window)
        peoplesprites.draw(window)
        pygame.display.flip()
    
            
main()
